package com.woolha.jpamulticolumnjoin.model;

import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.Id;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.JoinColumns;
import jakarta.persistence.OneToMany;
import jakarta.persistence.Table;
import jakarta.persistence.UniqueConstraint;
import java.math.BigDecimal;
import java.util.Set;
import java.util.UUID;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.id.uuid.UuidGenerator;

@AllArgsConstructor
@NoArgsConstructor
@Builder
@Getter
@Setter
@Entity
@Table(name = "products", uniqueConstraints = {
    @UniqueConstraint(name = "uq_products_category_id_code", columnNames = {"categoryId", "code"})
})
public class Product {

  @Id
  @GeneratedValue
  @GenericGenerator(name = "UUID", type = UuidGenerator.class)
  private UUID id;

  private UUID categoryId;

  private String code;

  private String name;

  private BigDecimal price;

  /**
   * If you want to join using native query (to avoid eager fetching), this field can be removed.
   */
  @JoinColumns({
      @JoinColumn(name ="categoryId", referencedColumnName = "categoryId", insertable = false, updatable = false),
      @JoinColumn(name ="productCode", referencedColumnName = "code", insertable = false, updatable = false),
  })
  @OneToMany
  //  @OneToMany(mappedBy = "product") // You can remove the @JoinColumns above and uncomment this line if the annotation is already defined in Order
  private Set<Order> orders;
}
