package com.woolha.jpamulticolumnjoin.repository;

import com.woolha.jpamulticolumnjoin.dto.OrderWithProductDto;
import com.woolha.jpamulticolumnjoin.dto.ProductDto;
import jakarta.persistence.EntityManager;
import jakarta.persistence.NoResultException;
import jakarta.persistence.PersistenceContext;
import jakarta.persistence.Query;
import java.math.BigDecimal;
import java.util.Optional;
import java.util.UUID;

public class CustomOrderRepositoryImpl implements CustomOrderRepository {

  @PersistenceContext
  private EntityManager entityManager;

  @Override
  public Optional<OrderWithProductDto> findOneWithProductById(UUID id) {
    String sql = "SELECT o.*, p FROM orders o INNER JOIN products p"
        + " ON o.category_id = p.category_id AND o.product_code = p.code"
        + " WHERE o.id = :id";

    @SuppressWarnings("unchecked")
    Query query = this.entityManager.createNativeQuery(sql)
        .unwrap(org.hibernate.query.Query.class)
        .setTupleTransformer(((tuple, aliases) -> {
          Object[] productData = (Object[]) tuple[5];
          return OrderWithProductDto.builder()
              .id((UUID) tuple[0])
              .quantity((int) tuple[1])
              .price((BigDecimal) tuple[2])
              .categoryId((UUID) tuple[3])
              .productCode((String) tuple[4])
              .product(ProductDto.builder()
                  .id(UUID.fromString((String) productData[0]))
                  .categoryId(UUID.fromString((String) productData[1]))
                  .code((String) productData[2])
                  .name((String) productData[3])
                  .price(new BigDecimal((String) productData[4]))
                  .build()
              )
              .build();
        }))
        .setParameter("id", id)
        .setMaxResults(1);

    try {
      OrderWithProductDto result = (OrderWithProductDto) query.getSingleResult();

      return Optional.of(result);
    } catch (NoResultException ex) {
      return Optional.empty();
    }
  }
}
