package com.woolha.jpamulticolumnjoin.repository;

import com.woolha.jpamulticolumnjoin.dto.OrderDto;
import com.woolha.jpamulticolumnjoin.dto.ProductWithOrderDto;
import com.woolha.jpamulticolumnjoin.dto.ProductWithOrdersDto;
import jakarta.persistence.EntityManager;
import jakarta.persistence.NoResultException;
import jakarta.persistence.PersistenceContext;
import jakarta.persistence.Query;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

public class CustomProductRepositoryImpl implements CustomProductRepository {

  @PersistenceContext
  private EntityManager entityManager;

  @Override
  public Optional<ProductWithOrdersDto> findOneWithOrdersById(UUID id) {
    String sql = "SELECT p.*, o FROM products p LEFT JOIN orders o"
        + " ON p.category_id = o.category_id AND p.code = o.product_code"
        + " WHERE p.id = :id";

    @SuppressWarnings("unchecked")
    Query query = this.entityManager.createNativeQuery(sql)
        .unwrap(org.hibernate.query.Query.class)
        .<ProductWithOrderDto>setTupleTransformer(((tuple, aliases) -> {
          Object[] productData = (Object[]) tuple[5];
          return ProductWithOrderDto.builder()
              .id((UUID) tuple[0])
              .categoryId((UUID) tuple[1])
              .code((String) tuple[2])
              .name((String) tuple[3])
              .price((BigDecimal) tuple[4])
              .order(OrderDto.builder()
                  .id(UUID.fromString((String) productData[0]))
                  .quantity(Integer.parseInt((String) productData[1]))
                  .price(new BigDecimal((String) productData[2]))
                  .categoryId(UUID.fromString((String) productData[3]))
                  .productCode((String) productData[4])
                  .build()
              )
              .build();
        }))
        .setParameter("id", id);

    try {
      @SuppressWarnings("unchecked") List<ProductWithOrderDto> productWithOrderList = (List<ProductWithOrderDto>) query.getResultList();

      ProductWithOrdersDto result = ProductWithOrdersDto.builder()
          .id(productWithOrderList.get(0).getId())
          .categoryId(productWithOrderList.get(0).getCategoryId())
          .code(productWithOrderList.get(0).getCode())
          .name(productWithOrderList.get(0).getName())
          .price(productWithOrderList.get(0).getPrice())
          .orders(productWithOrderList.stream()
              .map(ProductWithOrderDto::getOrder)
              .collect(Collectors.toList())
          )
          .build();

      return Optional.of(result);
    } catch (NoResultException ex) {
      return Optional.empty();
    }
  }
}
