package com.woolha.jpamulticolumnjoin.service;

import com.woolha.jpamulticolumnjoin.dto.OrderDto;
import com.woolha.jpamulticolumnjoin.dto.ProductWithOrdersDto;
import com.woolha.jpamulticolumnjoin.model.Order;
import com.woolha.jpamulticolumnjoin.model.Product;
import com.woolha.jpamulticolumnjoin.repository.ProductRepository;
import java.util.Optional;
import java.util.UUID;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

@RequiredArgsConstructor
@Service
public class ProductService {

  private final ProductRepository productRepository;

  public void fetch(UUID id) {
    Optional<Product> productOptional = this.productRepository.findById(id);

    if (productOptional.isEmpty()) {
      System.out.println("Not found");
      return;
    }

    Product product = productOptional.get();
    System.out.println("ID: " + product.getId());
    System.out.println("Category ID: " + product.getCategoryId());
    System.out.println("Code: " + product.getCode());
    System.out.println("Name: " + product.getName());
    System.out.println("Price: " + product.getPrice());

    for (Order order : product.getOrders()) {
      System.out.println("---------------------------------");
      System.out.println("Order - ID: " + order.getId());
      System.out.println("Order - Category ID: " + order.getCategoryId());
      System.out.println("Order - Product code: " + order.getProductCode());
      System.out.println("Order - Quantity: " + order.getQuantity());
      System.out.println("Order - Price: " + order.getPrice());
    }
  }

  public void fetchUsingNativeQuery(UUID id) {
    Optional<ProductWithOrdersDto> productOptional = this.productRepository.findOneWithOrdersById(id);

    if (productOptional.isEmpty()) {
      System.out.println("Not found");
      return;
    }

    ProductWithOrdersDto product = productOptional.get();
    System.out.println("ID: " + product.getId());
    System.out.println("Category ID: " + product.getCategoryId());
    System.out.println("Code: " + product.getCode());
    System.out.println("Name: " + product.getName());
    System.out.println("Price: " + product.getPrice());

    for (OrderDto order : product.getOrders()) {
      System.out.println("---------------------------------");
      System.out.println("Order - ID: " + order.getId());
      System.out.println("Order - Category ID: " + order.getCategoryId());
      System.out.println("Order - Product code: " + order.getProductCode());
      System.out.println("Order - Quantity: " + order.getQuantity());
      System.out.println("Order - Price: " + order.getPrice());
    }
  }
}
